#Param: simple routine for command line argument / config file processing
#Version 2.5.1

#Copyright (c) 2005 Thomas Orgis <thomas@orgis.org>
#This is Open Source, Distribution and Modification under the terms of the Artistic License

use strict;

package Param;

use Sys::Hostname;
use FindBin qw($Bin); #RealBin or just Bin?
use File::Spec;
use File::Basename;

#command line args parsing

#i predefine param{help} with switch h!!!
#i predefine param{config} with switch I!!!

sub Parse #(config, params)
{
	my $config = shift;
	my $printconfig = 0;
	#$config->{verbose} = 1;
	my $hh = 'show the help message (no value required...)';
	my $ih = 'Which configfile to use;'."\n".'special: just -I or --config causes printing a current config file to STDOUT';
	my %param = ('help',0, 'config', undef);	
	my %parexist = ('help',1,'config',1); #for checking for unknown parameters
	my %par_help = ('h',\$hh,'I',\$ih);
	my %par_map = ('h','help','I','config');
	my $length = 6;
	$param{config} = $config->{file};
	my $confdir = (defined $config->{confdir}) ? $config->{confdir} : File::Spec->catfile($Bin,'..','etc');
	print STDERR "Looking for config files in $confdir\n" if $config->{verbose};
	#means: nofile[false,true], file[string], info, verbose[bool],  
	#config confusion
	unless($config->{nofile} or $param{config}) #as long as I was told not to use a config file or it has been already given
	{
		#scan for a config file
		my @l = ( File::Spec->catfile($confdir,basename($0).'.conf'), File::Spec->catfile($confdir,basename($0).'.'.hostname().'.conf') );
		print STDERR "possible config files: @l\n" if $config->{verbose};
		if($config->{multi})
		{
			#well, this is a nice example of totally relying on a two-valued array
			shift(@l) unless (-f $l[0]); #remove default file from list when not existing
			pop(@l) unless (-f $l[$#l]); #remove hostfile from list when not existing
		}
		else
		{
			pop(@l) unless (-f $l[$#l]); #only when existing
			shift(@l) if ($#l == 1 or ! (-f $l[0])); #hehe...
		}
		$param{config} = join(',', @l); #to be compatible with command line input
	}
	#create parameter entries
	my $pars = [];
	my $args = [];
	if($#_ > 1 or $#_ == -1)
	{
		$pars = \@_;
		$args = \@ARGV;
	}
	else
	{
		$pars = shift;
		$args = defined $_[0] ? shift : \@ARGV;
	}
	my $r;
	my $t;
	die "Param: Wrong Parameter count. This is the Programmer's fault.\n" if (($#{$pars}+1) % 4) != 0;
	
	if($config->{gimme} and $#{$args}==-1){ $param{help} = 1} #giving help when no arguments
	
	while(defined ($r = shift(@{$pars})))
	{
		my ($val,$short,$help) = (shift(@{$pars}),shift(@{$pars}),\(shift(@{$pars})));
		unless(defined $param{$r} or defined $par_map{$short})
		{
				$length = length($r) if length($r) > $length;
				$param{$r} = $val;
				$parexist{$r} = 1;
				$par_map{$short} = $r;
				$par_help{$short} = $help;
		}
		else{print STDERR "NewPar: Tried to redefine an option ($r / $short)! Programmer: please check this!\n"; return -1;}
	}
	
	#now interpret command line opts / config file
	my %local_settings = ();
	
	#regexes for option parsing, the interesting part goes in $2!
	
	my $shorts = '^(-([^-].*))$'; #allow -x -x=bla -xyz
	my $longs = '^(--(.+))'; #allow --long --long=lala
	my $separator = '^--$'; #exactly this string means "Stop the parsing!"
	
	if($config->{lazy})
	{
		$shorts = '^(-?([^-]|[^-]=.*|[^-=]+))$'; #allows -x -x=bla x x=bla xyz -xyz
		$longs = '^(-{0,2}(.{2,}=.*)|--(.+))$'; #allows --long=bla long=bla -long=bla --long, not "long" only!
		$separator = '^-+$';
	}
	
	while(defined $args->[0])
	{
		if($args->[0] =~ /$shorts/)
		{
			#a short option or aggregate of those
			my $a = $2;
			if($a =~ /^(.)=(.*)$/) #a single -a=x
			{
				print STDERR "param $1\n" if $config->{verbose};
				#conffile immediately and only if value provided
				if($1 eq 'I'){ $param{config} = $2; }
				#normal processing
				elsif(defined $par_map{$1})
				{
					$local_settings{$par_map{$1}} = $2;
				}
				else{ $param{help} = 1; &ParError($1,\%par_map); } 
			}
			else
			{
				#collect
				for(my $i = 0;$i < length($a); ++$i)
				{
					if(defined (my $s = $par_map{substr($a,$i,1)}))
					{
						if($s ne 'config'){$local_settings{$s} = 1;}
						else{ $printconfig = 1; }
					}
					else{ $param{help} = 1; &ParError(substr($a,$i,1), \%par_map); }
				}
			}
			shift(@{$args});
		}
		else
		{
			#long option, presumably
		  if($args->[0] =~ /$longs/)	
			{
				#yeah, long option
				my $opt = defined $2 ? $2 : $3;
				if($opt =~ /^(.+)=(.*)$/)
				{
					#--long-option=value
					#conffile immediately and only if value provided
					if($1 eq 'config'){ $param{config} = $2; }
					elsif($parexist{$1}){ $local_settings{$1} = $2; }
					else{ $param{help} = 1; &ParError($1,\%param); }
				}
				elsif($parexist{$opt})
				{
					#just --long-option
					if($opt ne 'config'){$local_settings{$opt} = 1;}
					else{ $printconfig = 1; }
				} 
				else{ $param{help} = 1; &ParError($opt,\%param); } 
				shift(@{$args});
			}
			elsif($args->[0] =~ /$separator/)
			{
				shift(@{$args}); last; #got the definite delimiter: --, switch part over
			}
			else{ last; } #was no option... consider the switch part over
		}
	}

	#now parse config file
	if($param{config} ne "")
	{
		ReadFile($confdir, $config, \%param, \%parexist);
	}
	
	#and now apply the explicit command line switches,options,parameters,...
	foreach my $key (keys %local_settings)
	{
		$param{length($key) == 1 ? $par_map{$key} : $key} = $local_settings{$key};
	}

	#give the help (info text + option help) and exit when -h or --help was given
	if($param{help} and !$config->{ignorehelp})
	{
		print "\n",$config->{info},"\n\n" if defined $config->{info}; 
		print "Parameter input:\n\t", $config->{lazy} ? "[[-]optlist] [[-]<option>[=<value>]] [[[-]-]<long-option>=<value>] [-[-]<long-option>] [-[-] [other stuff]]\nThe complicated look of this line due to the many [] means just that you can save time and command line length in dropping the \"-\" prefixes (note: \"bla\" mentions three short options, not the long option), but in trade for that the separator for additional non-option parameters is mandatory (supply as many \"-\" grouped together as you like;-)." : "[-<optionlist>] [-<option>[=<value>]] [--<long-option>[=<value>]] [[--] [filenames or other stuff maybe needed]]" ,"\n";
		print "Btw.: An only mentioned short/long option (no value) means setting to 1, which is true in the logical sense.\n\n";
		#print "Without value the option is set to 1, which normally just means \"true\".\n\n";
		print "Recognized Options:\n";
		print "OPTION, LONG-OPTION [VALUE] DESCRIPTION\n";
		foreach my $k ( sort keys %par_map )
		{
			my @help = split("\n",${$par_help{$k}});
			$help[0] = '['.$param{$par_map{$k}}.'] '.$help[0];
			print "$k, $par_map{$k} ";
			my $i = length($par_map{$k}) + 4;
			foreach my $h (@help)
			{
				for(; $i < $length + 4; ++$i){ print " "; }
				$i = 0;
				print "$h\n";
			}
		}
		print "Hope I could help you. Bye.\n\n";
		exit(0);
	}

	if($printconfig)
	{
		PrintFile(\*STDOUT, $config, \%param, \%par_map, \%par_help);
		exit(0);
	}
	return $config->{returnall} ? (\%param,  \%par_map, \%par_help, \%parexist) : \%param;	
}

sub ParError
{
	my $par = shift;
	my $hash = shift;
	print STDERR "Unknown parameter/switch \"$par\" not in (",join(',',keys %{$hash}),")!\n";
}

sub ReadFile
{
	my ($confdir,$config, $param, $parexist) = @_;

	my $lend = '\012\015';
	my $unspecial = '[^\#=\s<\012\015]';
	
	my @l = ($config->{multi}) ? split(',',$param->{config}) : ($param->{config});
	foreach my $file (@l)
	{
		$file = File::Spec->catfile($confdir,$file) unless (-f $file); #look in confidir
		print STDERR "Parsing $file...\n" if $config->{verbose}; 
		if(open(CDAT,$file))
		{
			#do we need or want binmode for newlines?
			my $multiline = '';
			my $mcollect = 0;
			my $ender = '';
			my $mkey ='';
			while(<CDAT>)
			{
				unless($mcollect)
				{
					unless($_ =~ /^\s*#?\s*[$lend]*$/)
					{
						if($_ =~ /^\s*($unspecial+)\s*=\s*([^$lend]*)[$lend]*$/)
						{
							my ($par,$val) = ($1,$2);
							#remove trailing spaces
							$val =~ s/\s*$//;
							#remove quotes
							$val =~ s/^"(.*)"$/$1/;
							if(not defined $parexist or $parexist->{$par})
							{
								print STDERR "Setting $par to: $val ", ($config->{eval} ? "(evaluating first)" : ""), "\n" if $config->{verbose}; 
								$param->{$par} = ($config->{eval}) ? eval($val) : $val; #apply the config file options first, with eval() when desired
								$param->{$par} = '' unless defined $param->{$par};
							}
							else{ $param->{help} = 1; &ParError($par, $param);}
						}
						elsif($_ =~ /^\s*($unspecial+)\s*[$lend]*$/)
						{
							if(not defined $parexist or $parexist->{$1})
							{
								print STDERR "Setting $1 to 1\n" if $config->{verbose}; 
								$param->{$1} = 1;
							}
							else{ $param->{help} = 1; &ParError($1, $param); }
						}
						elsif($_ =~ /^\s*($unspecial+)\s*<<(.*)$/)
						{
							$ender = $2;
							$mkey = $1;
							$mcollect = 1;							
							print STDERR "Reading for $mkey...\n" if $config->{verbose}; 
						}
					}
				}
				else
				{
					unless($_ =~ /^$ender$/)
					{
						s/[$lend]+$/\n/;
						$multiline .= $_;
					}
					else
					{
						$mcollect = 0;
						if(not defined $parexist or $parexist->{$mkey})
						{
							$param->{$mkey} = ($config->{eval}) ? eval($multiline) : $multiline; #apply the config file options first, with eval() when desired
							#$param->{$mkey} = '' unless defined $param->{$mkey}; #really sensible here?
						}
						else{ $param->{help} = 1; &ParError($mkey, $param); }

						$multiline = '';
					}
				}
			}
			close(CDAT);
			print STDERR "...done parsing.\n" if $config->{verbose};
		}
		else{print STDERR "Couldn't open config file $file! ($!)\n";}
	}
}


#(
sub PrintFile
{
	my ($handle, $config, $param, $par_map, $par_help) = @_;
	print $handle <<EOT;
# Configuration file for $0
#
# Syntax:
# 
EOT
if($config->{eval})
{
	print $handle <<EOT;
# 	name = perl expression
#
# Enclosing "" are optional and you can provide any number (including 0) of whitespaces before and after name and the expression.
# The expression gets eval()ed in-place and the result is stored in the parameter named name. When you want to set a plain string, use double quoting (""string"" or "'string'", when you know Perl, you know the difference;-) or just single single quotes ('value').
EOT
}
else
{
	print $handle <<EOT;
# 	name = value
# or
# 	name = "value"
#
# You can provide any number (including 0) of whitespaces before and after name and value. If you really want the whitespace in the value then use the second form and be happy;-)
EOT
}
print $handle <<EOT;
# It is also possible to set multiline strings with
# name <<ENDSTRING
# ...
# ENDSTRING
#	
# (just like in PERL but omitting the ;)
# The short names are just provided as a reference; they're only working as real command line parameters, not in this file!
EOT
	foreach my $k (sort keys %{$par_map})
	{
		#make line ending changeable...
		#or use proper system-independent line end
		#for now we just use \n - what may even work with active perl
		#
		print $handle "\n# $par_map->{$k} (short: $k)\n"; 
		my @help = split("\n",${$par_help->{$k}});
		for(@help)
		{
			print $handle "# $_\n";
		}
		if($param->{$par_map->{$k}} =~ /\n/)
		{
			my $end = 'EOT';
			my $num = '';
			$param->{$par_map->{$k}} =~ s/[^\n]\z/\n/;
			while($param->{$par_map->{$k}} =~ /(^|\n)$end$num(\n|$)/){ ++$num; }
			print $handle "\n$par_map->{$k} <<$end$num\n$param->{$par_map->{$k}}$end$num\n";
		}
		elsif($config->{eval})
		{
			#Are numbers strings? They could be... perl sorts it out anyway.
			print $handle defined $param->{$par_map->{$k}} ? "\n$par_map->{$k} = '$param->{$par_map->{$k}}'\n" : "\n$par_map->{$k} = undef\n";
		}
		else
		{
			print $handle defined $param->{$par_map->{$k}} ? "\n$par_map->{$k} = \"$param->{$par_map->{$k}}\"\n" : "\n# $par_map->{$k} is undefined\n";
		}
	}
}

1;


__END__


=head1 NAME

Param - work with command line arguments and config files (yes, another one...)

=head1 SYNOPSIS

	use Param;
	#remember: parameters help / h	and config / I are predefined!
	$my_parameter_ref = Param::Parse
	(
	{
		'info' => 'This is the program info string that gets spitten out on `your-program -h` before the generic usage help'
	},
	"parm1",0,"a","the help text for the first parameter",
	"parm2",0,"b","the help text for the second parameter",
	...
	"parmX",0,"z","the help text for the last parameter"
	);
	print "The (probably changed by the command line) value of parameter 'parm1' is $my_parameter_ref->{parm1}\n";

or

	use Param;
	$my_parameter_ref = Param::Parse(\%config,\@params); 

or

	use Param;
	$my_parameter_ref = Param::Parse(\%config,\@params,\@commandline_args); 
	
=head1 DESCRIPTION

This module evolved from a simple loop for command line argument processing, that I thought was quicker written than to search for an already existing module for that. While just using Getopt I<may> have been a bit quicker, this wouldn't fully do what I want. It may fully do what I<you> want, though - so better check this before blaming me for wasting your time with installing Param.pm .

So, this is what I want and what therefore is provided by this module:

=over 4

=item * command line parameter processing

Process command line options/switches/parameters/... , be it short or long style, supporting clustering of short options. Interprets B<--parm=value> / B<-p=value> with the obvious effect; sets option to 1 (true) when just B<--option> / B<-o> is given.
The form "--parm value" is not supported as no simple bullet-proof generic way to do that came across my mind. There is the fundamental problem of deciding if we have a parameter value or some other command line data like a file name to process. Since this problem still persists with the "=" used in assignment when one considers a file with a name like "--i_look_like_an_option", which is perfectly possible, Param also looks out for "--" as a final delimiter for the option part (like startx distinguishes between client options and server options). After "--" the Parse routine touches nothing.
A propos touch: The parameters are taken from the global @ARGV or the provided array (third parameter, see third example), which is shifted on processing. So everything interpreted by Param is deleted afterwards.

=item * automatic usage message creation

Based on the parameter definition Param automatically prints the expected usage/help message when the (predefined!) B<--help> / B<-h> was given, with the info string in advance when	defined, and exits the program.

=item * configuration file parsing

The module also introduces a simple but flexible configuration file facility. Configuration means simply setting the same parameters that are available to the command line, with the added possibility to use perl syntax (expressions for eval(), the result being stored as value) when enabled in the program.
The second and last predefined parameter called "config" (short: I) is for giving a file (or several ones, see below!) to process I<before> the command line options are applied (what the user typed is always the last word). If none is given, some guessing may take place to find an appropriate file (see below).
When just -I or --config is given (no "="!), then a file with the current configuration ist written to STDOUT and the program exits.

=back

As you probably have read, the usage consists mainly of calling the routine Param::Parse() with the parameter definitions in a flat array following an anonymous hash / a reference or just with hash/array references for the same stuff. The second variant includes the possibility to specify an array to use instead of @ARGV (per reference). 
The config hash configures the behaviour of Param.pm itself. No value is mandatory, but setting of the info string is highly recommened. All values are undefined (empty, 0, false) per default.
The keys:

=over 4

=item * info (some text)

A text to print out before the parameter info when giving help.

=item * verbose (0 / 1)

When not 0, be verbose about the work being done. There may be different verbosity levels in the future, but for now value 4 is no different from 1.

=item * multi (0 / 1)

Enable parsing of several configuration files one after another

=item * confdir (some text)

Define a directory for searching of config files, overriding the internal default of "../etc" relative to the directory containing the program; explicitly given files are always searched in the current working directory first if the given form does not already specify an existing file completely. 

=item * file (some text)

Define the default config file (s) (when using multiple ones: "file1,file2,file3,..."), overriding the internal default of confdir/programname.conf or (being more powerful if existing) confdir/programname.hostname.conf .
A word on the hostname: This is a possibility to use one configuration directory for several hosts (p.ex. mounted over NFS). 

=item * eval (0/ 1)

Activate the parsing of config files with eval(), this can be dangerous, but great fun, too;-) One may even manipulate internals on Param.pm. I won't give a guide to that here... 

=item * gimme (0 / 1)

Give help end exit if no command line arguments are there.

=item * lazy (0 / 1)

Switch to the "lazy" syntax that allows omitting the "-" and "--" in -option and --long-option most of the times but absolutely requires the separation of other stuff with "-*" (read this as PERL regex;-).

=item * ignorehelp (0 / 1)

ignore the help switch (when you don't want Param to exit() your program in any case)

=item * returnall (0 / 1)

makes Parse() return not only the parameter hash but also other stuff that is needed by the wrapper module Param::FileWorker

=back

=head1 AUTHOR

	Thomas Orgis <thomas@orgis.org>

=cut


