#include "../include/common.h"

void integrate(ostream &of, ex &ap, ex &pp,
               double av, double pv, double step, long num)
{
	//ap = f(alpha,p)
	//pp = f(alpha,p)
	
	ex ka[4];
	ex kp[4];
	ex nexta;
	ex nextp;

	//Uebersichtlicherweise wird der Winkel in Vielfachen von Pi angegeben.
	//a = b*pi -> apunkt = bpunkt*pi -> bpunkt = apunkt/pi ...
	ex pi_ap = ap.subs(alpha == alpha*Pi) / Pi;
	ex pi_pp = pp.subs(alpha == alpha*Pi);

	//Dateikopf und Startpunkt
	of << "#alpha/pi	p_alpha" << endl;
	of << av << "	" << pv << endl; 

	//und nun num Punkte dazu
	for(int i = 0; i < num; ++i)
	{

		/*
		  Berechnung mit GiNaC-Ausdruecken.
		  Natuerlich is dies alles reichlich ineffizient, wenn wir an der hohen
			Genauigkeit gar nicht interessiert sind. Einfach double-Nummern werden
			sicherlich schneller mit einer "einfachen" C-Funktion erzeugt, jedoch
			muesste man dafuer diese Funktion erzeugen, was automatisch nicht so
			einfach ist.
			Somit ist es fuer den Einstieg schneller, die Operationen
			brav symbolisch anzulassen.
		*/

		ka[0] = (ex)step * pi_ap.subs(lst( alpha == av, p == pv ));
		kp[0] = (ex)step * pi_pp.subs(lst( alpha == av, p == pv ));

		ka[1] = (ex)step * 
		        pi_ap.subs(lst( alpha == av + ka[0] / 2, p == pv + kp[0] / 2 ));
		kp[1] = (ex)step *
		        pi_pp.subs(lst( alpha == av + ka[0] / 2, p == pv + kp[0] / 2 ));

		ka[2] = (ex)step *
		        pi_ap.subs(lst( alpha == av + ka[1] / 2, p == pv + kp[1] / 2 ));
		kp[2] = (ex)step *
		        pi_pp.subs(lst( alpha == av + ka[1] / 2, p == pv + kp[1] / 2 ));

		ka[3] = (ex)step * pi_ap.subs(lst( alpha == av + ka[2], p == pv + kp[2] ));
		kp[3] = (ex)step * pi_pp.subs(lst( alpha == av + ka[2], p == pv + kp[2] ));
		
		nexta = av + ( ka[0] + 2*ka[1] + 2*ka[2] + ka[3]) / 6;
		nextp = pv + ( kp[0] + 2*kp[1] + 2*kp[2] + kp[3]) / 6;
		
		/*
			Ein evtl. enthaltenes Pi wird mittels evalf() neutralisiert, bevor eine
			numeric-Nummer daraus erzeugt werden kann!
			Dann einfache Auswertung als double.
			Hier werfen wir einen groen Haufen Genauigkeit weg!
		*/
		av = ex_to<numeric>(nexta.evalf()).to_double();
		pv = ex_to<numeric>(nextp.evalf()).to_double();		
		
		/*
			Ebenfalls gehe ich von einem Winkel alpha aus, in dem das System auch
			periodisch ist.
			Soll heien: -Pi bis +Pi reicht mir.
		*/
		of << ( av > 1 ?
		        av - (long)((av+1)/2)*2 : 
		        ( av < -1 ? av - (long)((av-1)/2)*2 : av)
		      ) << "	" << pv << endl; 
	}
}
