#include "include/common.h"
#include "include/system.h"
#include "include/poisson.h"
#include "include/totex.h"
#include "include/betrachter.h"

ex apunkt;
ex ppunkt;
ex pot;

int usage(char* argv[]);
void vollauto(char* argv[]);
void halbauto(char* argv[]);
void einzel(char* argv[]);

int main(int argc, char* argv[])
{
	pot = poti();
	ex ham = hamilton();

	if( (argc != 5) && (argc != 7) && (argc != 8) ) return usage(argv);

	//Aufstellen der Bewegungsgleichungen ueber die Poissonklammer 
	//und etwas Dissipation
	double diss = atof(argv[1]);
	apunkt = poisson(alpha,ham);
	ppunkt = poisson(p,ham) - diss*apunkt;
	
	if(argc != 8)
	{
		//Grundlegende Feststellung am Anfang
		cout << "Potential: " << pot << endl;
		totex(pot,"potential.tex");
		cout << "Hamilton: " << ham << endl;
		totex(ham,"hamilton.tex");
		//Ausgabe der Gleichungen
		cout << "apunkt = " << apunkt << endl << "ppunkt = " << ppunkt << endl;
		totex(apunkt, "apunkt.tex");
		totex(ppunkt, "ppunkt.tex");
		ofstream parf("rechner.par");
		parf << "diss = " << argv[1] << endl
		     << "step = " << argv[2] << endl
		     << "points = " << argv[3] << endl
		     << "starts = " << argv[4] << endl;
		parf.close();
	}

	//argc kann nur noch 5,7 oder 8 sein!
	if(argc == 8)	einzel(argv);
	else
	{
		if(argc == 7) halbauto(argv);
		else vollauto(argv);
	}

	return 0;
}

int usage(char* argv[])
{
	cout << "usage: " << argv[0] << " diss step points starts" << endl
	     << " for many trajectories for some predefined combinations "
	     << "of Omega and A"	<< endl	<< " or " << endl
	     << "       " << argv[0] << " diss step points starts Omega A" << endl
	     << " for many trajectories for this combination" << endl
	     << " or " << endl
	     << "       " << argv[0] << " diss step points Omega A starta"
	     << " startp" << endl
	     << " for one trajectory printed to stdout" << endl;
	return 1;
}

void vollauto(char* argv[])
{
	double step = atof(argv[2]);
	long points = atol(argv[3]);
	long starts = atol(argv[4]);
	
	//Betrachtungen fuer die verschiedenen Parameterkombinationen
	for(int i = 0; i <= 4; ++i)
	{
		betrachter(pot, apunkt, ppunkt,0.5*i, 0, step, points, starts);
		betrachter(pot, apunkt, ppunkt,0.5*i, 0.3, step, points, starts);
		betrachter(pot, apunkt, ppunkt,0.5*i, 3, step, points, starts);
	}
}

void halbauto(char* argv[])
{
	double step = atof(argv[2]);
	long points = atol(argv[3]);
	long starts = atol(argv[4]);
	double Omega = atof(argv[5]);
	double A = atof(argv[6]);

	//Betrachtung fuer die eine Parameterkombination
	betrachter(pot, apunkt, ppunkt, Omega, A, step, points, starts);
}

void einzel(char* argv[])
{
	double step = atof(argv[2]);
	long points = atol(argv[3]);
	double Omega = atof(argv[4]);
	double A = atof(argv[5]);
	double starta = atof(argv[6]);
	double startp = atof(argv[7]);
	
	betrachter(apunkt, ppunkt, Omega, A, step, points, starta, startp);
}

